@@ -48,7 +48,6 @@
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
 #include "empathy-contact-menu.h"
-#include "empathy-gtk-marshal.h"
 #include "empathy-search-bar.h"
 #include "empathy-theme-manager.h"
 #include "empathy-smiley-manager.h"
@@ -58,7 +57,6 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
-
 #define CHAT_DIR_CREATE_MODE  (S_IRUSR | S_IWUSR | S_IXUSR)
 #define CHAT_FILE_CREATE_MODE (S_IRUSR | S_IWUSR)
 #define IS_ENTER(v) (v == GDK_KEY_Return || v == GDK_KEY_ISO_Enter || v == GDK_KEY_KP_Enter)
@@ -631,45 +629,37 @@ typedef struct {
 	gchar *message;
 } ChatCommandMsgData;
 
+#if 0
 static void
-chat_command_msg_cb (GObject *source,
-			      GAsyncResult *result,
+chat_command_msg_cb (EmpathyDispatchOperation *dispatch,
+			      const GError             *error,
 			      gpointer                  user_data)
 {
 	ChatCommandMsgData *data = user_data;
-	GError *error = NULL;
-	TpChannel *channel;
-
-	channel = tp_account_channel_request_ensure_and_observe_channel_finish (
-					TP_ACCOUNT_CHANNEL_REQUEST (source), result, &error);
-
-	if (channel == NULL) {
-		DEBUG ("Failed to get channel: %s", error->message);
-		g_error_free (error);
 
+	if (error != NULL) {
 		empathy_chat_view_append_event (data->chat->view,
 			_("Failed to open private chat"));
 		goto OUT;
 	}
 
-	if (!EMP_STR_EMPTY (data->message) && TP_IS_TEXT_CHANNEL (channel)) {
-		TpTextChannel *text = (TpTextChannel *) channel;
-		TpMessage *msg;
+	if (!EMP_STR_EMPTY (data->message)) {
+		EmpathyTpChat *tpchat;
+		EmpathyMessage *message;
 
-		msg = tp_client_message_new_text (TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
-			data->message);
+		tpchat = EMPATHY_TP_CHAT (
+			empathy_dispatch_operation_get_channel_wrapper (dispatch));
 
-		tp_text_channel_send_message_async (text, msg, 0, NULL, NULL);
-
-		g_object_unref (msg);
+		message = empathy_message_new (data->message);
+		empathy_tp_chat_send (tpchat, message);
+		g_object_unref (message);
 	}
 
-	g_object_unref (channel);
-
 OUT:
 	g_free (data->message);
 	g_slice_free (ChatCommandMsgData, data);
 }
+#endif
 
 static void
 chat_command_clear (EmpathyChat *chat,
@@ -731,36 +721,26 @@ chat_command_join (EmpathyChat *chat,
 	g_strfreev (rooms);
 }
 
+#if 0
 static void
 chat_command_msg_internal (EmpathyChat *chat,
 			   const gchar *contact_id,
 			   const gchar *message)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
+	TpConnection *connection;
 	ChatCommandMsgData *data;
-	TpAccountChannelRequest *req;
-	GHashTable *request;
-
-	request = tp_asv_new (
-		TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_TEXT,
-		TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
-		TP_PROP_CHANNEL_TARGET_ID, G_TYPE_STRING, contact_id,
-		NULL);
-
-	req = tp_account_channel_request_new (priv->account, request,
-		tp_user_action_time_from_x11 (gtk_get_current_event_time ()));
 
 	/* FIXME: We should probably search in members alias. But this
 	 * is enough for IRC */
 	data = g_slice_new (ChatCommandMsgData);
 	data->chat = chat;
 	data->message = g_strdup (message);
-
-	tp_account_channel_request_ensure_and_observe_channel_async (req,
-		NULL, NULL, chat_command_msg_cb, data);
-
-	g_object_unref (req);
-	g_hash_table_unref (request);
+	connection = empathy_tp_chat_get_connection (priv->tp_chat);
+	empathy_dispatcher_chat_with_contact_id (connection, contact_id,
+						 gtk_get_current_event_time (),
+						 chat_command_msg_cb,
+						 data);
 }
 
 static void
@@ -778,6 +758,7 @@ chat_command_msg (EmpathyChat *chat,
 {
 	chat_command_msg_internal (chat, strv[1], strv[2]);
 }
+#endif
 
 static void
 chat_command_nick (EmpathyChat *chat,
@@ -849,11 +830,14 @@ static ChatCommandItem commands[] = {
 	{"j", 2, 2, chat_command_join,
 	 N_("/j <chat room ID>: join a new chat room")},
 
+#if 0
+/* FIXME: https://bugzilla.gnome.org/show_bug.cgi?id=623682 */
 	{"query", 2, 3, chat_command_query,
 	 N_("/query <contact ID> [<message>]: open a private chat")},
 
 	{"msg", 3, 3, chat_command_msg,
 	 N_("/msg <contact ID> <message>: open a private chat")},
+#endif
 
 	{"nick", 2, 2, chat_command_nick,
 	 N_("/nick <nickname>: change your nickname on the current server")},
@@ -1124,9 +1108,7 @@ chat_state_changed_cb (EmpathyTpChat      *tp_chat,
 }
 
 static void
-chat_message_received (EmpathyChat *chat,
-	EmpathyMessage *message,
-	gboolean pending)
+chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	EmpathyContact  *sender;
@@ -1145,7 +1127,7 @@ chat_message_received (EmpathyChat *chat,
 			       chat);
 
 	priv->unread_messages++;
-	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message, pending);
+	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message);
 }
 
 static void
@@ -1153,7 +1135,7 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 			  EmpathyMessage *message,
 			  EmpathyChat    *chat)
 {
-	chat_message_received (chat, message, FALSE);
+	chat_message_received (chat, message);
 }
 
 static void
@@ -1781,33 +1763,79 @@ chat_spelling_menu_activate_cb (GtkMenuItem     *menu_item,
                                gtk_menu_item_get_label (menu_item));
 }
 
+
+static GtkWidget *
+chat_spelling_build_suggestions_menu (const gchar *code,
+				      EmpathyChatSpell *chat_spell)
+{
+	GList     *suggestions, *l;
+	GtkWidget *menu, *menu_item;
+
+	suggestions = empathy_spell_get_suggestions (code, chat_spell->word);
+	if (suggestions == NULL)
+		return NULL;
+
+	menu = gtk_menu_new ();
+	for (l = suggestions; l; l = l->next) {
+		menu_item = gtk_menu_item_new_with_label (l->data);
+		g_signal_connect (G_OBJECT (menu_item), "activate",
+				  G_CALLBACK (chat_spelling_menu_activate_cb),
+				  chat_spell);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	}
+	empathy_spell_free_suggestions (suggestions);
+
+	gtk_widget_show_all (menu);
+
+	return menu;
+}
+
 static GtkWidget *
 chat_spelling_build_menu (EmpathyChatSpell *chat_spell)
 {
-    GtkWidget *menu, *menu_item;
-    GList     *suggestions, *l;
+	GtkWidget *menu, *submenu, *item;
+	GList     *codes, *l;
 
-    menu = gtk_menu_new ();
-    suggestions = empathy_spell_get_suggestions (chat_spell->word);
-    if (suggestions == NULL) {
-        menu_item = gtk_menu_item_new_with_label (_("(No Suggestions)"));
-	gtk_widget_set_sensitive (menu_item, FALSE);
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-    } else {
-        for (l = suggestions; l; l = l->next) {
-            menu_item = gtk_menu_item_new_with_label (l->data);
-            g_signal_connect (G_OBJECT (menu_item),
-                          "activate",
-                          G_CALLBACK (chat_spelling_menu_activate_cb),
-                          chat_spell);
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-        }
-    }
-    empathy_spell_free_suggestions (suggestions);
+	codes = empathy_spell_get_enabled_language_codes ();
+	g_assert (codes != NULL);
 
-    gtk_widget_show_all (menu);
+	if (g_list_length (codes) > 1) {
+		menu = gtk_menu_new ();
 
-    return menu;
+		for (l = codes; l; l = l->next) {
+			const gchar *code, *name;
+
+			code = l->data;
+			name = empathy_spell_get_language_name (code);
+			if (!name)
+				continue;
+
+			item = gtk_image_menu_item_new_with_label (name);
+
+			submenu = chat_spelling_build_suggestions_menu (
+					code, chat_spell);
+			if (submenu == NULL)
+				gtk_widget_set_sensitive (item, FALSE);
+			else
+				gtk_menu_item_set_submenu (GTK_MENU_ITEM (item),
+							   submenu);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		}
+	} else {
+		menu = chat_spelling_build_suggestions_menu (codes->data,
+							     chat_spell);
+		if (menu == NULL) {
+			menu = gtk_menu_new ();
+			item = gtk_menu_item_new_with_label (_("(No Suggestions)"));
+			gtk_widget_set_sensitive (item, FALSE);
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		}
+	}
+	g_list_free (codes);
+
+	gtk_widget_show_all (menu);
+
+	return menu;
 }
 
 static void
@@ -1911,7 +1939,7 @@ chat_input_populate_popup_cb (GtkTextView *view,
 
 
 static gboolean
-chat_log_filter (TplEvent *log,
+chat_log_filter (TplEntry *log,
 		 gpointer user_data)
 {
 	EmpathyChat *chat = user_data;
@@ -1919,15 +1947,14 @@ chat_log_filter (TplEvent *log,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	const GList *pending;
 
-	g_return_val_if_fail (TPL_IS_EVENT (log), FALSE);
+	g_return_val_if_fail (TPL_IS_ENTRY (log), FALSE);
 	g_return_val_if_fail (EMPATHY_IS_CHAT (chat), FALSE);
 
 	pending = empathy_tp_chat_get_pending_messages (priv->tp_chat);
-	message = empathy_message_from_tpl_log_event (log);
+	message = empathy_message_from_tpl_log_entry (log);
 
 	for (; pending; pending = g_list_next (pending)) {
 		if (empathy_message_equal (message, pending->data)) {
-			g_object_unref (message);
 			return FALSE;
 		}
 	}
@@ -1954,7 +1981,7 @@ show_pending_messages (EmpathyChat *chat) {
 
 	for (l = messages; l != NULL ; l = g_list_next (l)) {
 		EmpathyMessage *message = EMPATHY_MESSAGE (l->data);
-		chat_message_received (chat, message, TRUE);
+		chat_message_received (chat, message);
 	}
 }
 
@@ -1970,7 +1997,7 @@ got_filtered_messages_cb (GObject *manager,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	GError *error = NULL;
 
-	if (!tpl_log_manager_get_filtered_events_finish (TPL_LOG_MANAGER (manager),
+	if (!tpl_log_manager_get_filtered_messages_finish (TPL_LOG_MANAGER (manager),
 		result, &messages, &error)) {
 		DEBUG ("%s. Aborting.", error->message);
 		empathy_chat_view_append_event (chat->view,
@@ -1981,9 +2008,9 @@ got_filtered_messages_cb (GObject *manager,
 
 	for (l = messages; l; l = g_list_next (l)) {
 		EmpathyMessage *message;
-		g_assert (TPL_IS_EVENT (l->data));
+		g_assert (TPL_IS_ENTRY (l->data));
 
-		message = empathy_message_from_tpl_log_event (l->data);
+		message = empathy_message_from_tpl_log_entry (l->data);
 		g_object_unref (l->data);
 
 		empathy_chat_view_append_message (chat->view, message);
@@ -2009,7 +2036,7 @@ static void
 chat_add_logs (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	TplEntity       *target;
+	gboolean         is_chatroom;
 
 	if (!priv->id) {
 		return;
@@ -2019,23 +2046,18 @@ chat_add_logs (EmpathyChat *chat)
 	empathy_chat_view_scroll (chat->view, FALSE);
 
 	/* Add messages from last conversation */
-	if (priv->handle_type == TP_HANDLE_TYPE_ROOM)
-	  target = tpl_entity_new_from_room_id (priv->id);
-	else
-	  target = tpl_entity_new (priv->id, TPL_ENTITY_CONTACT, NULL, NULL);
+	is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
 
 	priv->retrieving_backlogs = TRUE;
-	tpl_log_manager_get_filtered_events_async (priv->log_manager,
-						   priv->account,
-						   target,
-						   TPL_EVENT_MASK_TEXT,
-						   5,
-						   chat_log_filter,
-						   chat,
-						   got_filtered_messages_cb,
-						   (gpointer) chat);
-
-	g_object_unref (target);
+	tpl_log_manager_get_filtered_messages_async (priv->log_manager,
+							      priv->account,
+							      priv->id,
+							      is_chatroom,
+							      5,
+							      chat_log_filter,
+							      chat,
+							      got_filtered_messages_cb,
+							      (gpointer) chat);
 }
 
 static gint
@@ -2445,7 +2467,6 @@ chat_create_ui (EmpathyChat *chat)
 	gchar           *filename;
 	GtkTextBuffer   *buffer;
 	gint              paned_pos;
-	EmpathyThemeManager *theme_mgr;
 
 	filename = empathy_file_lookup ("empathy-chat.ui",
 					"libempathy-gtk");
@@ -2471,9 +2492,7 @@ chat_create_ui (EmpathyChat *chat)
 	g_free (filename);
 
 	/* Add message view. */
-	theme_mgr = empathy_theme_manager_dup_singleton ();
-	chat->view = empathy_theme_manager_create_view (theme_mgr);
-	g_object_unref (theme_mgr);
+	chat->view = empathy_theme_manager_create_view (empathy_theme_manager_get ());
 	/* If this is a GtkTextView, it's set as a drag destination for text/plain
 	   and other types, even though it's non-editable and doesn't accept any
 	   drags.  This steals drag motion for anything inside the scrollbars,
@@ -2562,29 +2581,6 @@ chat_create_ui (EmpathyChat *chat)
 	g_object_unref (gui);
 }
 
-static void
-chat_size_request (GtkWidget      *widget,
-		   GtkRequisition *requisition)
-{
-  GtkBin *bin = GTK_BIN (widget);
-  GtkWidget *child;
-
-  requisition->width = gtk_container_get_border_width (GTK_CONTAINER (widget)) * 2;
-  requisition->height = gtk_container_get_border_width (GTK_CONTAINER (widget)) * 2;
-
-  child = gtk_bin_get_child (bin);
-
-  if (child && gtk_widget_get_visible (child))
-    {
-      GtkRequisition child_requisition;
-
-      gtk_widget_size_request (child, &child_requisition);
-
-      requisition->width += child_requisition.width;
-      requisition->height += child_requisition.height;
-    }
-}
-
 static void
 chat_size_allocate (GtkWidget     *widget,
 		    GtkAllocation *allocation)
@@ -2701,7 +2697,6 @@ empathy_chat_class_init (EmpathyChatClass *klass)
 	object_class->set_property = chat_set_property;
 	object_class->constructed = chat_constructed;
 
-	widget_class->size_request = chat_size_request;
 	widget_class->size_allocate = chat_size_allocate;
 
 	g_object_class_install_property (object_class,
@@ -2778,9 +2773,9 @@ empathy_chat_class_init (EmpathyChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      _empathy_gtk_marshal_VOID__OBJECT_BOOLEAN,
+			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      2, EMPATHY_TYPE_MESSAGE, G_TYPE_BOOLEAN);
+			      1, EMPATHY_TYPE_MESSAGE);
 
 	g_type_class_add_private (object_class, sizeof (EmpathyChatPriv));
 }
@@ -3193,8 +3188,7 @@ empathy_chat_get_contact_menu (EmpathyChat *chat)
 		menu = empathy_contact_menu_new (priv->remote_contact,
 						 EMPATHY_CONTACT_FEATURE_CALL |
 						 EMPATHY_CONTACT_FEATURE_LOG |
-						 EMPATHY_CONTACT_FEATURE_INFO |
-						 EMPATHY_CONTACT_FEATURE_BLOCK);
+						 EMPATHY_CONTACT_FEATURE_INFO);
 	}
 	else if (priv->contact_list_view) {
 		EmpathyContactListView *view;
