@@ -50,7 +50,6 @@
 #include "empathy-groups-widget.h"
 #include "empathy-ui-utils.h"
 #include "empathy-string-parser.h"
-#include "empathy-kludge-label.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include <libempathy/empathy-debug.h>
@@ -586,11 +585,11 @@ contact_widget_groups_update (EmpathyContactWidget *information)
       FolksPersona *persona =
           empathy_contact_get_persona (information->contact);
 
-      if (FOLKS_IS_GROUP_DETAILS (persona))
+      if (FOLKS_IS_GROUPABLE (persona))
         {
-          empathy_groups_widget_set_group_details (
+          empathy_groups_widget_set_groupable (
               EMPATHY_GROUPS_WIDGET (information->groups_widget),
-              FOLKS_GROUP_DETAILS (persona));
+              FOLKS_GROUPABLE (persona));
           gtk_widget_show (information->groups_widget);
 
           return;
@@ -1284,10 +1283,10 @@ contact_widget_contact_update (EmpathyContactWidget *information)
           FolksPersona *persona = empathy_contact_get_persona (
               information->contact);
 
-          if (persona != NULL && FOLKS_IS_FAVOURITE_DETAILS (persona))
+          if (persona != NULL && FOLKS_IS_FAVOURITE (persona))
             {
-              gboolean is_favourite = folks_favourite_details_get_is_favourite (
-                  FOLKS_FAVOURITE_DETAILS (persona));
+              gboolean is_favourite = folks_favourite_get_is_favourite (
+                  FOLKS_FAVOURITE (persona));
               contact_widget_favourites_changed_cb (information->manager,
                   information->contact, is_favourite, information);
             }
@@ -1418,19 +1417,17 @@ favourite_toggled_cb (GtkToggleButton *button,
 {
   FolksPersona *persona = empathy_contact_get_persona (information->contact);
 
-  if (persona != NULL && FOLKS_IS_FAVOURITE_DETAILS (persona))
+  if (persona != NULL && FOLKS_IS_FAVOURITE (persona))
     {
       gboolean active = gtk_toggle_button_get_active (button);
-      folks_favourite_details_set_is_favourite (
-          FOLKS_FAVOURITE_DETAILS (persona), active);
+      folks_favourite_set_is_favourite (FOLKS_FAVOURITE (persona), active);
     }
 }
 
 static void
 contact_widget_contact_setup (EmpathyContactWidget *information)
 {
-  /* Setup label_status as a KludgeLabel */
-  information->label_status = empathy_kludge_label_new ("");
+  information->label_status = gtk_label_new ("");
   gtk_label_set_line_wrap_mode (GTK_LABEL (information->label_status),
                                 PANGO_WRAP_WORD_CHAR);
   gtk_label_set_line_wrap (GTK_LABEL (information->label_status),
