@@ -1,6 +1,6 @@
 /*
  * empathy-call-window.c - Source for EmpathyCallWindow
- * Copyright (C) 2008-2011 Collabora Ltd.
+ * Copyright (C) 2008-2009 Collabora Ltd.
  * @author Sjoerd Simons <sjoerd.simons@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
@@ -30,15 +30,19 @@
 #include <glib/gi18n.h>
 
 #include <telepathy-glib/util.h>
-#include <telepathy-farstream/telepathy-farstream.h>
+#include <telepathy-farsight/channel.h>
 #include <telepathy-glib/util.h>
 
 #include <gst/farsight/fs-element-added-notifier.h>
-#include <gst/farsight/fs-utils.h>
 
 #include <libempathy/empathy-tp-contact-factory.h>
+#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy-gtk/empathy-avatar-image.h>
+#include <libempathy-gtk/empathy-video-widget.h>
+#include <libempathy-gtk/empathy-audio-src.h>
+#include <libempathy-gtk/empathy-audio-sink.h>
+#include <libempathy-gtk/empathy-video-src.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-sound.h>
 #include <libempathy-gtk/empathy-geometry.h>
@@ -49,12 +53,7 @@
 
 #include "empathy-call-window.h"
 #include "empathy-call-window-fullscreen.h"
-#include "empathy-call-factory.h"
-#include "empathy-video-widget.h"
-#include "empathy-audio-src.h"
-#include "empathy-audio-sink.h"
-#include "empathy-video-src.h"
-#include "empathy-sidebar.h"
+#include "ev-sidebar.h"
 
 #define BUTTON_ID "empathy-call-dtmf-button-id"
 
@@ -79,6 +78,16 @@
 
 G_DEFINE_TYPE(EmpathyCallWindow, empathy_call_window, GTK_TYPE_WINDOW)
 
+/* signal enum */
+#if 0
+enum
+{
+    LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = {0};
+#endif
+
 enum {
   PROP_CALL_HANDLER = 1,
 };
@@ -96,11 +105,13 @@ typedef enum {
   CAMERA_STATE_ON,
 } CameraState;
 
+/* private structure */
+typedef struct _EmpathyCallWindowPriv EmpathyCallWindowPriv;
+
 struct _EmpathyCallWindowPriv
 {
   gboolean dispose_has_run;
   EmpathyCallHandler *handler;
-
   EmpathyContact *contact;
 
   guint call_state;
@@ -124,7 +135,8 @@ struct _EmpathyCallWindowPriv
   GtkWidget *pane;
   GtkAction *redial;
   GtkAction *menu_fullscreen;
-  GtkAction *action_camera_on;
+  GtkAction *action_camera;
+  GtkAction *action_camera_preview;
   GtkWidget *tool_button_camera_off;
   GtkWidget *tool_button_camera_preview;
   GtkWidget *tool_button_camera_on;
@@ -180,7 +192,7 @@ struct _EmpathyCallWindowPriv
   GstElement *funnel;
   GstElement *liveadder;
 
-  GList *notifiers;
+  FsElementAddedNotifier *fsnotifier;
 
   guint context_id;
 
@@ -211,7 +223,9 @@ struct _EmpathyCallWindowPriv
   gboolean pipeline_playing;
 };
 
-#define GET_PRIV(o) (EMPATHY_CALL_WINDOW (o)->priv)
+#define GET_PRIV(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((o), EMPATHY_TYPE_CALL_WINDOW, \
+    EmpathyCallWindowPriv))
 
 static void empathy_call_window_realized_cb (GtkWidget *widget,
   EmpathyCallWindow *window);
@@ -231,10 +245,10 @@ static void empathy_call_window_set_send_video (EmpathyCallWindow *window,
 static void empathy_call_window_mic_toggled_cb (
   GtkToggleToolButton *toggle, EmpathyCallWindow *window);
 
-static void empathy_call_window_sidebar_hidden_cb (EmpathySidebar *sidebar,
+static void empathy_call_window_sidebar_hidden_cb (EvSidebar *sidebar,
   EmpathyCallWindow *window);
 
-static void empathy_call_window_sidebar_shown_cb (EmpathySidebar *sidebar,
+static void empathy_call_window_sidebar_shown_cb (EvSidebar *sidebar,
   EmpathyCallWindow *window);
 
 static void empathy_call_window_hangup_cb (gpointer object,
@@ -265,6 +279,9 @@ static void empathy_call_window_restart_call (EmpathyCallWindow *window);
 static void empathy_call_window_status_message (EmpathyCallWindow *window,
   gchar *message);
 
+static void empathy_call_window_update_avatars_visibility (EmpathyTpCall *call,
+  EmpathyCallWindow *window);
+
 static gboolean empathy_call_window_bus_message (GstBus *bus,
   GstMessage *message, gpointer user_data);
 
@@ -324,17 +341,17 @@ static void
 dtmf_button_pressed_cb (GtkButton *button, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
   GQuark button_quark;
   TpDTMFEvent event;
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
 
   button_quark = g_quark_from_static_string (BUTTON_ID);
   event = GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (button),
     button_quark));
 
-  tpy_call_channel_dtmf_start_tone (call, event);
+  empathy_tp_call_start_tone (call, event);
 
   g_object_unref (call);
 }
@@ -343,11 +360,11 @@ static void
 dtmf_button_released_cb (GtkButton *button, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
 
-  tpy_call_channel_dtmf_stop_tone (call);
+  empathy_tp_call_stop_tone (call);
 
   g_object_unref (call);
 }
@@ -586,9 +603,8 @@ empathy_call_window_create_audio_input (EmpathyCallWindow *self)
 
   priv->volume_progress_bar = gtk_progress_bar_new ();
 
-  gtk_progress_bar_set_orientation (
-    GTK_PROGRESS_BAR (priv->volume_progress_bar),
-      GTK_PROGRESS_BOTTOM_TO_TOP);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (priv->volume_progress_bar),
+      GTK_ORIENTATION_VERTICAL);
 
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (priv->volume_progress_bar),
       0);
@@ -821,7 +837,7 @@ disable_camera (EmpathyCallWindow *self)
 
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (
       priv->tool_button_camera_off), TRUE);
-  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera_on),
+  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera),
       CAMERA_STATE_OFF);
   unblock_camera_control_signals (self);
 
@@ -878,7 +894,7 @@ enable_preview (EmpathyCallWindow *self)
 
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (
         priv->tool_button_camera_preview), TRUE);
-  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera_on),
+  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera),
       CAMERA_STATE_PREVIEW);
   unblock_camera_control_signals (self);
 
@@ -934,7 +950,7 @@ enable_camera (EmpathyCallWindow *self)
 
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (
       priv->tool_button_camera_on), TRUE);
-  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera_on),
+  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera),
       CAMERA_STATE_ON);
   unblock_camera_control_signals (self);
 
@@ -1013,16 +1029,15 @@ create_pipeline (EmpathyCallWindow *self)
 static void
 empathy_call_window_init (EmpathyCallWindow *self)
 {
-  EmpathyCallWindowPriv *priv;
+  EmpathyCallWindowPriv *priv = GET_PRIV (self);
   GtkBuilder *gui;
   GtkWidget *top_vbox;
   GtkWidget *h;
   GtkWidget *arrow;
   GtkWidget *page;
   gchar *filename;
-
-  priv = self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-    EMPATHY_TYPE_CALL_WINDOW, EmpathyCallWindowPriv);
+  GKeyFile *keyfile;
+  GError *error = NULL;
 
   filename = empathy_file_lookup ("empathy-call-window.ui", "src");
   gui = empathy_builder_get_file (filename,
@@ -1039,7 +1054,8 @@ empathy_call_window_init (EmpathyCallWindow *self)
     "camera_off", &priv->tool_button_camera_off,
     "camera_preview", &priv->tool_button_camera_preview,
     "camera_on", &priv->tool_button_camera_on,
-    "action_camera_on",  &priv->action_camera_on,
+    "action_camera_off",  &priv->action_camera,
+    "action_camera_preview",  &priv->action_camera_preview,
     "details_vbox",  &priv->details_vbox,
     "vcodec_encoding_label", &priv->vcodec_encoding_label,
     "acodec_encoding_label", &priv->acodec_encoding_label,
@@ -1066,7 +1082,7 @@ empathy_call_window_init (EmpathyCallWindow *self)
     "camera_off", "toggled", tool_button_camera_off_toggled_cb,
     "camera_preview", "toggled", tool_button_camera_preview_toggled_cb,
     "camera_on", "toggled", tool_button_camera_on_toggled_cb,
-    "action_camera_on", "changed", action_camera_change_cb,
+    "action_camera_off", "changed", action_camera_change_cb,
     NULL);
 
   gtk_action_set_sensitive (priv->menu_fullscreen, FALSE);
@@ -1118,9 +1134,27 @@ empathy_call_window_init (EmpathyCallWindow *self)
   create_audio_output (self);
   create_video_input (self);
 
+  priv->fsnotifier = fs_element_added_notifier_new ();
+  fs_element_added_notifier_add (priv->fsnotifier, GST_BIN (priv->pipeline));
+
   /* The call will be started as soon the pipeline is playing */
   priv->start_call_when_playing = TRUE;
 
+  keyfile = g_key_file_new ();
+  filename = empathy_file_lookup ("element-properties", "data");
+  if (g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, &error))
+    {
+      fs_element_added_notifier_set_properties_from_keyfile (priv->fsnotifier,
+          keyfile);
+    }
+  else
+    {
+      g_warning ("Could not load element-properties file: %s", error->message);
+      g_key_file_free (keyfile);
+      g_clear_error (&error);
+    }
+  g_free (filename);
+
   priv->vbox = gtk_vbox_new (FALSE, 3);
   gtk_box_pack_start (GTK_BOX (priv->content_hbox), priv->vbox,
       FALSE, FALSE, CONTENT_HBOX_CHILDREN_PACKING_PADDING);
@@ -1140,7 +1174,7 @@ empathy_call_window_init (EmpathyCallWindow *self)
   gtk_box_pack_end (GTK_BOX (priv->vbox), h, FALSE, FALSE, 3);
   gtk_box_pack_end (GTK_BOX (h), priv->sidebar_button, FALSE, FALSE, 3);
 
-  priv->sidebar = empathy_sidebar_new ();
+  priv->sidebar = ev_sidebar_new ();
   g_signal_connect (G_OBJECT (priv->sidebar),
     "hide", G_CALLBACK (empathy_call_window_sidebar_hidden_cb), self);
   g_signal_connect (G_OBJECT (priv->sidebar),
@@ -1148,20 +1182,20 @@ empathy_call_window_init (EmpathyCallWindow *self)
   gtk_paned_pack2 (GTK_PANED (priv->pane), priv->sidebar, FALSE, FALSE);
 
   page = empathy_call_window_create_audio_input (self);
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+  ev_sidebar_add_page (EV_SIDEBAR (priv->sidebar), "audio-input",
       _("Audio input"), page);
 
   page = empathy_call_window_create_video_input (self);
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+  ev_sidebar_add_page (EV_SIDEBAR (priv->sidebar), "video-input",
       _("Video input"), page);
 
   priv->dtmf_panel = empathy_call_window_create_dtmf (self);
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+  ev_sidebar_add_page (EV_SIDEBAR (priv->sidebar), "dialpad",
       _("Dialpad"), priv->dtmf_panel);
 
   gtk_widget_set_sensitive (priv->dtmf_panel, FALSE);
 
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+  ev_sidebar_add_page (EV_SIDEBAR (priv->sidebar), "details",
       _("Details"), priv->details_vbox);
 
   gtk_widget_show_all (top_vbox);
@@ -1227,19 +1261,12 @@ set_window_title (EmpathyCallWindow *self)
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
   gchar *tmp;
 
-  if (priv->contact != NULL)
-    {
-      /* translators: Call is a noun and %s is the contact name. This string
-       * is used in the window title */
-      tmp = g_strdup_printf (_("Call with %s"),
-          empathy_contact_get_alias (priv->contact));
-      gtk_window_set_title (GTK_WINDOW (self), tmp);
-      g_free (tmp);
-    }
-  else
-    {
-      gtk_window_set_title (GTK_WINDOW (self), _("Call with %d participants"));
-    }
+  /* translators: Call is a noun and %s is the contact name. This string
+   * is used in the window title */
+  tmp = g_strdup_printf (_("Call with %s"),
+      empathy_contact_get_alias (priv->contact));
+  gtk_window_set_title (GTK_WINDOW (self), tmp);
+  g_free (tmp);
 }
 
 static void
@@ -1289,22 +1316,40 @@ empathy_call_window_setup_avatars (EmpathyCallWindow *self,
     EmpathyCallHandler *handler)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  TpConnection *connection;
 
-  g_signal_connect (priv->contact, "notify::name",
-      G_CALLBACK (contact_name_changed_cb), self);
-  g_signal_connect (priv->contact, "notify::avatar",
-    G_CALLBACK (contact_avatar_changed_cb),
-    priv->remote_user_avatar_widget);
+  g_object_get (handler, "contact", &(priv->contact), NULL);
 
-  /* Retrieving the self avatar */
-  connection = empathy_contact_get_connection (priv->contact);
-  empathy_tp_contact_factory_get_from_handle (connection,
-      tp_connection_get_self_handle (connection),
-      empathy_call_window_got_self_contact_cb, self, NULL,
-      G_OBJECT (self));
+  if (priv->contact != NULL)
+    {
+      TpConnection *connection;
 
-  set_window_title (self);
+      set_window_title (self);
+
+      g_signal_connect (priv->contact, "notify::name",
+          G_CALLBACK (contact_name_changed_cb), self);
+      g_signal_connect (priv->contact, "notify::avatar",
+          G_CALLBACK (contact_avatar_changed_cb),
+          priv->remote_user_avatar_widget);
+
+      /* Retreiving the self avatar */
+      connection = empathy_contact_get_connection (priv->contact);
+      empathy_tp_contact_factory_get_from_handle (connection,
+          tp_connection_get_self_handle (connection),
+          empathy_call_window_got_self_contact_cb, self, NULL, G_OBJECT (self));
+    }
+  else
+    {
+      g_warning ("call handler doesn't have a contact");
+      /* translators: Call is a noun. This string is used in the window
+       * title */
+      gtk_window_set_title (GTK_WINDOW (self), _("Call"));
+
+      /* Since we can't access the remote contact, we can't get a connection
+         to it and can't get the self contact (and its avatar). This means
+         that we have to manually set the self avatar. */
+      init_contact_avatar_with_size (NULL, priv->self_user_avatar_widget,
+          MIN (SELF_VIDEO_SECTION_WIDTH, SELF_VIDEO_SECTION_HEIGTH));
+    }
 
   init_contact_avatar_with_size (priv->contact,
       priv->remote_user_avatar_widget,
@@ -1530,18 +1575,15 @@ empathy_call_window_constructed (GObject *object)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (object);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
 
   g_assert (priv->handler != NULL);
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
   priv->outgoing = (call == NULL);
   if (call != NULL)
     g_object_unref (call);
 
-  g_object_get (priv->handler, "target-contact", &priv->contact, NULL);
-  g_assert (priv->contact != NULL);
-
   empathy_call_window_setup_avatars (self, priv->handler);
   empathy_call_window_set_state_connecting (self);
 
@@ -1631,9 +1673,18 @@ empathy_call_window_class_init (
     PROP_CALL_HANDLER, param_spec);
 }
 
+static void
+empathy_call_window_video_stream_changed_cb (EmpathyTpCall *call,
+    GParamSpec *property, EmpathyCallWindow *self)
+{
+  DEBUG ("video stream changed");
+  empathy_call_window_update_avatars_visibility (call, self);
+}
+
 void
 empathy_call_window_dispose (GObject *object)
 {
+  EmpathyTpCall *call;
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (object);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
 
@@ -1642,11 +1693,19 @@ empathy_call_window_dispose (GObject *object)
 
   priv->dispose_has_run = TRUE;
 
+  g_object_get (priv->handler, "tp-call", &call, NULL);
+
+  if (call != NULL)
+    {
+      g_object_unref (call);
+    }
+
   if (priv->handler != NULL)
     {
       empathy_call_handler_stop_call (priv->handler);
-      tp_clear_object (&priv->handler);
+      g_object_unref (priv->handler);
     }
+  priv->handler = NULL;
 
   if (priv->bus_message_source_id != 0)
     {
@@ -1654,33 +1713,57 @@ empathy_call_window_dispose (GObject *object)
       priv->bus_message_source_id = 0;
     }
 
-  tp_clear_object (&priv->pipeline);
-  tp_clear_object (&priv->video_input);
-  tp_clear_object (&priv->audio_input);
-  tp_clear_object (&priv->audio_output);
-  tp_clear_object (&priv->video_tee);
-  tp_clear_object (&priv->ui_manager);
-  tp_clear_object (&priv->fullscreen);
+  if (priv->pipeline != NULL)
+    g_object_unref (priv->pipeline);
+  priv->pipeline = NULL;
 
-  g_list_free_full (priv->notifiers, g_object_unref);
+  if (priv->video_input != NULL)
+    g_object_unref (priv->video_input);
+  priv->video_input = NULL;
+
+  if (priv->audio_input != NULL)
+    g_object_unref (priv->audio_input);
+  priv->audio_input = NULL;
+
+  if (priv->audio_output != NULL)
+    g_object_unref (priv->audio_output);
+  priv->audio_output = NULL;
+
+  if (priv->video_tee != NULL)
+    g_object_unref (priv->video_tee);
+  priv->video_tee = NULL;
 
   if (priv->liveadder != NULL)
     gst_object_unref (priv->liveadder);
   priv->liveadder = NULL;
 
+  if (priv->fsnotifier != NULL)
+    g_object_unref (priv->fsnotifier);
+  priv->fsnotifier = NULL;
+
   if (priv->timer_id != 0)
     g_source_remove (priv->timer_id);
   priv->timer_id = 0;
 
+  if (priv->ui_manager != NULL)
+    g_object_unref (priv->ui_manager);
+  priv->ui_manager = NULL;
+
+  if (priv->fullscreen != NULL)
+    g_object_unref (priv->fullscreen);
+  priv->fullscreen = NULL;
+
   if (priv->contact != NULL)
     {
       g_signal_handlers_disconnect_by_func (priv->contact,
           contact_name_changed_cb, self);
+      g_object_unref (priv->contact);
       priv->contact = NULL;
     }
 
-
-  G_OBJECT_CLASS (empathy_call_window_parent_class)->dispose (object);
+  /* release any references held by the object here */
+  if (G_OBJECT_CLASS (empathy_call_window_parent_class)->dispose)
+    G_OBJECT_CLASS (empathy_call_window_parent_class)->dispose (object);
 }
 
 static void
@@ -1726,35 +1809,27 @@ empathy_call_window_conference_added_cb (EmpathyCallHandler *handler,
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  FsElementAddedNotifier *notifier;
-  GKeyFile *keyfile;
-
-  g_debug ("Conference added");
-
-  /* Add notifier to set the various element properties as needed */
-  notifier = fs_element_added_notifier_new ();
-  keyfile = fs_utils_get_default_element_properties (conference);
-
-  if (keyfile != NULL)
-    fs_element_added_notifier_set_properties_from_keyfile (notifier, keyfile);
-
-  fs_element_added_notifier_add (notifier, GST_BIN (priv->pipeline));
-
-  priv->notifiers = g_list_prepend (priv->notifiers, notifier);
 
   gst_bin_add (GST_BIN (priv->pipeline), conference);
+
   gst_element_set_state (conference, GST_STATE_PLAYING);
 }
 
-static void
-empathy_call_window_conference_removed_cb (EmpathyCallHandler *handler,
-  GstElement *conference, gpointer user_data)
+static gboolean
+empathy_call_window_request_resource_cb (EmpathyCallHandler *handler,
+  FsMediaType type, FsStreamDirection direction, gpointer user_data)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
 
-  gst_bin_remove (GST_BIN (priv->pipeline), conference);
-  gst_element_set_state (conference, GST_STATE_NULL);
+  if (type != FS_MEDIA_TYPE_VIDEO)
+    return TRUE;
+
+  if (direction == FS_DIRECTION_RECV)
+    return TRUE;
+
+  /* video and direction is send */
+  return priv->video_input != NULL;
 }
 
 static gboolean
@@ -1867,7 +1942,6 @@ empathy_call_window_disconnected (EmpathyCallWindow *self,
 
       /* Unsensitive the camera and mic button */
       gtk_widget_set_sensitive (priv->tool_button_camera_on, FALSE);
-      gtk_action_set_sensitive (priv->action_camera_on, FALSE);
       gtk_widget_set_sensitive (priv->mic_button, FALSE);
 
       /* Be sure that the mic button is enabled */
@@ -1923,15 +1997,16 @@ empathy_call_window_channel_closed_cb (EmpathyCallHandler *handler,
       empathy_call_window_restart_call (self);
 }
 
-static gboolean
-empathy_call_window_sink_removed_cb (EmpathyCallHandler *handler,
-    GstPad *sink,
-    FsMediaType media_type,
-    EmpathyCallWindow *self)
-{
-  EmpathyCallWindowPriv *priv = GET_PRIV (self);
 
-  g_print ("window: removing content\n");
+static void
+empathy_call_window_channel_stream_closed_cb (EmpathyCallHandler *handler,
+    TfStream *stream, gpointer user_data)
+{
+  EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
+  EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  guint media_type;
+
+  g_object_get (stream, "media-type", &media_type, NULL);
 
   /*
    * This assumes that there is only one video stream per channel...
@@ -1952,7 +2027,6 @@ empathy_call_window_sink_removed_cb (EmpathyCallHandler *handler,
           gst_bin_remove (GST_BIN (priv->pipeline), output);
           gst_bin_remove (GST_BIN (priv->pipeline), priv->funnel);
           priv->funnel = NULL;
-          return TRUE;
         }
     }
   else if (media_type == TP_MEDIA_STREAM_TYPE_AUDIO)
@@ -1965,11 +2039,8 @@ empathy_call_window_sink_removed_cb (EmpathyCallHandler *handler,
           gst_bin_remove (GST_BIN (priv->pipeline), priv->audio_output);
           gst_bin_remove (GST_BIN (priv->pipeline), priv->liveadder);
           priv->liveadder = NULL;
-          return TRUE;
         }
     }
-
-  return FALSE;
 }
 
 /* Called with global lock held */
@@ -2188,10 +2259,9 @@ empathy_call_window_update_timer (gpointer user_data)
   return TRUE;
 }
 
-#if 0
 static void
 display_error (EmpathyCallWindow *self,
-    TpyCallChannel *call,
+    EmpathyTpCall *call,
     const gchar *img,
     const gchar *title,
     const gchar *desc,
@@ -2266,12 +2336,12 @@ display_error (EmpathyCallWindow *self,
 
 static gchar *
 media_stream_error_to_txt (EmpathyCallWindow *self,
-    TpyCallChannel *call,
+    EmpathyTpCall *call,
     gboolean audio,
     TpMediaStreamError error)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  const gchar *cm = NULL;
+  const gchar *cm;
   gchar *url;
   gchar *result;
 
@@ -2308,9 +2378,7 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
                 "are not installed on your computer"));
 
       case TP_MEDIA_STREAM_ERROR_INVALID_CM_BEHAVIOR:
-        tp_connection_parse_object_path (
-            tp_channel_borrow_connection (TP_CHANNEL (call)),
-            NULL, &cm);
+        cm = empathy_tp_call_get_connection_manager (call);
 
         url = g_strdup_printf ("http://bugs.freedesktop.org/enter_bug.cgi?"
             "product=Telepathy&amp;component=%s", cm);
@@ -2321,7 +2389,6 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
               "logs gathered from the 'Debug' window in the Help menu."), url);
 
         g_free (url);
-        g_free (cm);
         return result;
 
       case TP_MEDIA_STREAM_ERROR_MEDIA_ERROR:
@@ -2338,7 +2405,7 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
 
 static void
 empathy_call_window_stream_error (EmpathyCallWindow *self,
-    TpyCallChannel *call,
+    EmpathyTpCall *call,
     gboolean audio,
     guint code,
     const gchar *msg,
@@ -2362,7 +2429,7 @@ empathy_call_window_stream_error (EmpathyCallWindow *self,
 }
 
 static void
-empathy_call_window_audio_stream_error (TpyCallChannel *call,
+empathy_call_window_audio_stream_error (EmpathyTpCall *call,
     guint code,
     const gchar *msg,
     EmpathyCallWindow *self)
@@ -2372,7 +2439,7 @@ empathy_call_window_audio_stream_error (TpyCallChannel *call,
 }
 
 static void
-empathy_call_window_video_stream_error (TpyCallChannel *call,
+empathy_call_window_video_stream_error (EmpathyTpCall *call,
     guint code,
     const gchar *msg,
     EmpathyCallWindow *self)
@@ -2380,46 +2447,46 @@ empathy_call_window_video_stream_error (TpyCallChannel *call,
   empathy_call_window_stream_error (self, call, FALSE, code, msg,
       "camera-web", _("Can't establish video stream"));
 }
-#endif
 
 static gboolean
 empathy_call_window_connected (gpointer user_data)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
   gboolean can_send_video;
 
   empathy_sound_stop (EMPATHY_SOUND_PHONE_OUTGOING);
 
-
-  can_send_video = priv->video_input != NULL &&
+  can_send_video = priv->video_input != NULL && priv->contact != NULL &&
     empathy_contact_can_voip_video (priv->contact);
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
 
-  if (tpy_call_channel_has_dtmf (call))
+  tp_g_signal_connect_object (call, "notify::video-stream",
+    G_CALLBACK (empathy_call_window_video_stream_changed_cb),
+    self, 0);
+
+  if (empathy_tp_call_has_dtmf (call))
     gtk_widget_set_sensitive (priv->dtmf_panel, TRUE);
 
   if (priv->video_input == NULL)
     empathy_call_window_set_send_video (self, CAMERA_STATE_OFF);
 
-  priv->sending_video = can_send_video &&
-    empathy_call_handler_has_initial_video (priv->handler);
+  priv->sending_video = can_send_video ?
+    empathy_tp_call_is_sending_video (call) : FALSE;
 
   gtk_toggle_tool_button_set_active (
       GTK_TOGGLE_TOOL_BUTTON (priv->tool_button_camera_on),
       priv->sending_video && priv->video_input != NULL);
   gtk_widget_set_sensitive (priv->tool_button_camera_on, can_send_video);
-  gtk_action_set_sensitive (priv->action_camera_on, can_send_video);
 
   gtk_action_set_sensitive (priv->redial, FALSE);
   gtk_widget_set_sensitive (priv->redial_button, FALSE);
 
   gtk_widget_set_sensitive (priv->mic_button, TRUE);
 
-  gtk_widget_hide (priv->video_output);
-  gtk_widget_show (priv->remote_user_avatar_widget);
+  empathy_call_window_update_avatars_visibility (call, self);
 
   g_object_unref (call);
 
@@ -2437,16 +2504,6 @@ empathy_call_window_connected (gpointer user_data)
   return FALSE;
 }
 
-static gboolean
-emapthy_call_window_show_video_output_cb (gpointer user_data)
-{
-  EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
-
-  gtk_widget_hide (self->priv->remote_user_avatar_widget);
-  gtk_widget_show (self->priv->video_output);
-
-  return FALSE;
-}
 
 /* Called from the streaming thread */
 static gboolean
@@ -2474,7 +2531,8 @@ empathy_call_window_src_added_cb (EmpathyCallHandler *handler,
         pad = empathy_call_window_get_audio_sink_pad (self);
         break;
       case TP_MEDIA_STREAM_TYPE_VIDEO:
-        g_idle_add (emapthy_call_window_show_video_output_cb, self);
+        gtk_widget_hide (priv->remote_user_avatar_widget);
+        gtk_widget_show (priv->video_output);
         pad = empathy_call_window_get_video_sink_pad (self);
         break;
       default:
@@ -2531,7 +2589,7 @@ empathy_call_window_src_added_cb (EmpathyCallHandler *handler,
 
 static gboolean
 empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
-  GstPad *sink, FsMediaType media_type, gpointer user_data)
+  GstPad *sink, guint media_type, gpointer user_data)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
@@ -2540,7 +2598,7 @@ empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
 
   switch (media_type)
     {
-      case FS_MEDIA_TYPE_AUDIO:
+      case TP_MEDIA_STREAM_TYPE_AUDIO:
         if (!gst_bin_add (GST_BIN (priv->pipeline), priv->audio_input))
           {
             g_warning ("Could not add audio source to pipeline");
@@ -2572,7 +2630,7 @@ empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
 
         retval = TRUE;
         break;
-      case FS_MEDIA_TYPE_VIDEO:
+      case TP_MEDIA_STREAM_TYPE_VIDEO:
         if (priv->video_input != NULL)
           {
             if (priv->video_tee != NULL)
@@ -2580,7 +2638,7 @@ empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
                 pad = gst_element_get_request_pad (priv->video_tee, "src%d");
                 if (GST_PAD_LINK_FAILED (gst_pad_link (pad, sink)))
                   {
-                    g_warning ("Could not link video source input pipeline");
+                    g_warning ("Could not link videp soure input pipeline");
                     break;
                   }
                 gst_object_unref (pad);
@@ -2623,7 +2681,6 @@ empathy_call_window_remove_video_input (EmpathyCallWindow *self)
   priv->video_preview = NULL;
 
   gtk_widget_set_sensitive (priv->tool_button_camera_on, FALSE);
-  gtk_action_set_sensitive (priv->action_camera_on, FALSE);
   gtk_widget_set_sensitive (priv->tool_button_camera_preview, FALSE);
 }
 
@@ -2730,23 +2787,40 @@ empathy_call_window_bus_message (GstBus *bus, GstMessage *message,
 }
 
 static void
-call_handler_notify_call_cb (EmpathyCallHandler *handler,
+empathy_call_window_update_avatars_visibility (EmpathyTpCall *call,
+    EmpathyCallWindow *window)
+{
+  EmpathyCallWindowPriv *priv = GET_PRIV (window);
+
+  if (empathy_tp_call_is_receiving_video (call))
+    {
+      gtk_widget_hide (priv->remote_user_avatar_widget);
+      gtk_widget_show (priv->video_output);
+    }
+  else
+    {
+      gtk_widget_hide (priv->video_output);
+      gtk_widget_show (priv->remote_user_avatar_widget);
+    }
+}
+
+static void
+call_handler_notify_tp_call_cb (EmpathyCallHandler *handler,
     GParamSpec *spec,
     EmpathyCallWindow *self)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
   if (call == NULL)
     return;
 
-/* FIXME
   tp_g_signal_connect_object (call, "audio-stream-error",
       G_CALLBACK (empathy_call_window_audio_stream_error), self, 0);
   tp_g_signal_connect_object (call, "video-stream-error",
       G_CALLBACK (empathy_call_window_video_stream_error), self, 0);
-*/
+
   g_object_unref (call);
 }
 
@@ -2754,40 +2828,39 @@ static void
 empathy_call_window_realized_cb (GtkWidget *widget, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
 
   g_signal_connect (priv->handler, "conference-added",
     G_CALLBACK (empathy_call_window_conference_added_cb), window);
-  g_signal_connect (priv->handler, "conference-removed",
-    G_CALLBACK (empathy_call_window_conference_removed_cb), window);
+  g_signal_connect (priv->handler, "request-resource",
+    G_CALLBACK (empathy_call_window_request_resource_cb), window);
   g_signal_connect (priv->handler, "closed",
     G_CALLBACK (empathy_call_window_channel_closed_cb), window);
   g_signal_connect (priv->handler, "src-pad-added",
     G_CALLBACK (empathy_call_window_src_added_cb), window);
   g_signal_connect (priv->handler, "sink-pad-added",
     G_CALLBACK (empathy_call_window_sink_added_cb), window);
-  g_signal_connect (priv->handler, "sink-pad-removed",
-    G_CALLBACK (empathy_call_window_sink_removed_cb), window);
+  g_signal_connect (priv->handler, "stream-closed",
+    G_CALLBACK (empathy_call_window_channel_stream_closed_cb), window);
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
   if (call != NULL)
     {
-/* FIXME: part of the improvements for DRAFT2
       tp_g_signal_connect_object (call, "audio-stream-error",
         G_CALLBACK (empathy_call_window_audio_stream_error), window,
         0);
       tp_g_signal_connect_object (call, "video-stream-error",
         G_CALLBACK (empathy_call_window_video_stream_error), window,
         0);
-*/
+
       g_object_unref (call);
     }
   else
     {
-      /* call-channel doesn't exist yet, we'll connect signals once it has been
+      /* tp-call doesn't exist yet, we'll connect signals once it has been
        * set */
-      g_signal_connect (priv->handler, "notify::call-channel",
-        G_CALLBACK (call_handler_notify_call_cb), window);
+      g_signal_connect (priv->handler, "notify::tp-call",
+        G_CALLBACK (call_handler_notify_tp_call_cb), window);
     }
 
   gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
@@ -2964,7 +3037,7 @@ empathy_call_window_set_send_video (EmpathyCallWindow *window,
   CameraState state)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  TpyCallChannel *call;
+  EmpathyTpCall *call;
 
   priv->sending_video = (state == CAMERA_STATE_ON);
 
@@ -2983,9 +3056,9 @@ empathy_call_window_set_send_video (EmpathyCallWindow *window,
   if (priv->call_state != CONNECTED)
     return;
 
-  g_object_get (priv->handler, "call-channel", &call, NULL);
+  g_object_get (priv->handler, "tp-call", &call, NULL);
   DEBUG ("%s sending video", priv->sending_video ? "start": "stop");
-  tpy_call_channel_send_video (call, priv->sending_video);
+  empathy_tp_call_request_video_stream_direction (call, priv->sending_video);
   g_object_unref (call);
 }
 
@@ -3018,7 +3091,7 @@ empathy_call_window_mic_toggled_cb (GtkToggleToolButton *toggle,
 }
 
 static void
-empathy_call_window_sidebar_hidden_cb (EmpathySidebar *sidebar,
+empathy_call_window_sidebar_hidden_cb (EvSidebar *sidebar,
   EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
@@ -3028,7 +3101,7 @@ empathy_call_window_sidebar_hidden_cb (EmpathySidebar *sidebar,
 }
 
 static void
-empathy_call_window_sidebar_shown_cb (EmpathySidebar *sidebar,
+empathy_call_window_sidebar_shown_cb (EvSidebar *sidebar,
   EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
@@ -3222,7 +3295,7 @@ block_camera_control_signals (EmpathyCallWindow *self)
       tool_button_camera_preview_toggled_cb, self);
   g_signal_handlers_block_by_func (priv->tool_button_camera_on,
       tool_button_camera_on_toggled_cb, self);
-  g_signal_handlers_block_by_func (priv->action_camera_on,
+  g_signal_handlers_block_by_func (priv->action_camera,
       action_camera_change_cb, self);
 }
 
@@ -3237,6 +3310,6 @@ unblock_camera_control_signals (EmpathyCallWindow *self)
       tool_button_camera_preview_toggled_cb, self);
   g_signal_handlers_unblock_by_func (priv->tool_button_camera_on,
       tool_button_camera_on_toggled_cb, self);
-  g_signal_handlers_unblock_by_func (priv->action_camera_on,
+  g_signal_handlers_unblock_by_func (priv->action_camera,
       action_camera_change_cb, self);
 }
